#!perl -w
# ====================================================================
# The Apache Software License, Version 1.1
#
# Copyright (c) 2003-2004 The Apache Software Foundation. All rights
# reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The end-user documentation included with the redistribution,
#    if any, must include the following acknowledgment:
#       "This product includes software developed by the
#        Apache Software Foundation (http://www.apache.org/)."
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. The names "Apache" and "Apache Software Foundation" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact apache@apache.org.
#
# 5. Products derived from this software may not be called "Apache",
#    nor may "Apache" appear in their name, without prior written
#    permission of the Apache Software Foundation.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# ====================================================================
#
# This software consists of voluntary contributions made by many
# individuals on behalf of the Apache Software Foundation.  For more
# information on the Apache Software Foundation, please see
# <http://www.apache.org/>.
#

# This script is intended to be called from within the docs build process.
# It gets two parameters, ${input_ext} (e.g. .xml.de) and the full path name
# of the current file.
# It assumes, that the current working directory is the root of the manual.

# The script doesn't use an XML parser (yet?) so care should be taken with
# non-ascii characters.

# TODO:
# - rewrite in java as ant task?

# against fat fingers
use strict;

# for file operations
use FindBin;
use lib $FindBin::Bin;
use Fcntl qw(O_RDONLY O_WRONLY O_CREAT O_TRUNC);
use DocUtil qw(%suffix2lang $ignore_files srcinfo docpath);

# scratch stuff
my (%current, %variants, @pfiles, $curpath);

# check commandline parameters
die "missing parameter(s)\n"
    unless defined $ARGV[0] and defined $ARGV[1];
    #              suffix               srcfile

my ($dirname, $basename, $relative, $metafile) = srcinfo();

# first determine the reality
$curpath = docpath($dirname);
opendir(DIR, $curpath)
    or die "could not open directory '$curpath' ($!), stopped";

@pfiles = sort
       grep /^\Q$basename.\E(?:html\.[^.]+|xml)/
    => grep !/$ignore_files/
    => readdir(DIR);

closedir(DIR)
    or die "could not close directory '$curpath' ($!), stopped";

# now loop over the filelist.
# it contains all files of one language including
# html AND xml variants.
for (@pfiles) {
    my $suffix;
    my $lang = $suffix = /^\Q$basename.\E(?:html|xml)\.([^.]+)/ ? $1 : 'en';
    $lang = $suffix2lang{$suffix} if $suffix2lang{$suffix};

    $variants{$lang} = {};
    $variants{$lang}->{'pdf'} = "$basename.$suffix.pdf"
        if -f docpath("$dirname$basename.$suffix.pdf");

    # the following works, because the list (@pfiles) is sorted, that way
    # html is processed before xml and thus typically we set the
    # parameter first and delete it later.
    if (/^\Q$basename.xml/) {
        delete $variants{$lang}->{'htmlonly'};
    }
    else {
        $variants{$lang}->{'htmlonly'} = 'yes';
    }
}

# create resulting file contents
my $content = <<"XML";
<?xml version="1.0" encoding="UTF-8" ?>

<metafile>
  <basename>$basename</basename>
  <path>$dirname</path>
  <relpath>$relative</relpath>

  <variants>
    @{[
        join "\n    "
        => map
            qq{<variant@{[do{ my $lang = $_;
                map qq( $_="$variants{$lang}->{$_}")
                => keys %{$variants{$lang}};
            }]}>$_</variant>}
        => sort keys %variants
    ]}
  </variants>
</metafile>
XML

# read current variant file if exists
my $existing = '';
$curpath = docpath($metafile);
if (-f $curpath) {
    local $/; # slurp mode /

    sysopen(FILE, $curpath, O_RDONLY)
        or die "could not open file '$curpath' ($!), stopped";

    $existing = <FILE>;

    close(FILE)
        or die "could not close file '$curpath' ($!), stopped";
}

# compare and possibly write new result to disk
unless ($content eq $existing) {
    my $ppath = $metafile;
    $ppath =~ s,^/,,;

    sysopen(FILE, $curpath, O_WRONLY | O_CREAT | O_TRUNC)
        or die "could not open file '$curpath' ($!), stopped";

    print FILE $content
        or die "could not write into file '$curpath' ($!), stopped";

    close(FILE)
        or die "could not close file '$curpath' ($!), stopped";

    # report success
    print "'$ppath' written.\n";
}

__END__
sample file:

<?xml version="1.0" encoding="UTF-8" ?>

<metafile>
  <basename>quickreference</basename>
  <path>/mod/</path>
  <relpath>..</relpath>

  <variants>
    <variant pdf="quickreference.en.pdf">en</variant>
    <variant>ja</variant>
  </variants>
</metafile>
